import { describe, it, expect } from 'vitest';
import {
  updateArrayStats,
  updateBooleanStats,
  updateCategoricalStats,
  updateNumericalStats,
  updateTemporalStats,
  updateObjectStats,
  initializeStatistics,
} from '../utils/attributeStats';
import type { ArrayStats, BooleanStats, CategoricalStats, NumericalStats, TemporalStats, ObjectStats } from 'ts-common/src/model/query/statistics';

describe('updateArrayStats', () => {
  it('should update the length of the array', () => {
    const stats: ArrayStats = { length: 0, count: 0 };
    const value = [1, 2, 3];
    updateArrayStats(stats, value);
    expect(stats.length).toBe(3);
    expect(stats.count).toBe(1);
  });
});

describe('updateBooleanStats', () => {
  it('should update true count when value is true', () => {
    const stats: BooleanStats = { true: 0, false: 0 };
    updateBooleanStats(stats, true);
    expect(stats.true).toBe(1);
    expect(stats.false).toBe(0);
  });

  it('should update false count when value is false', () => {
    const stats: BooleanStats = { true: 0, false: 0 };
    updateBooleanStats(stats, false);
    expect(stats.false).toBe(1);
    expect(stats.true).toBe(0);
  });
});

describe('updateCategoricalStats', () => {
  it('should update mode and unique items count', () => {
    const stats: CategoricalStats = { uniqueItems: 0, values: [], mode: '', count: 0 };
    updateCategoricalStats(stats, 'apple');
    updateCategoricalStats(stats, 'banana');
    updateCategoricalStats(stats, 'apple');

    expect(stats.values).toEqual(['apple', 'banana', 'apple']);
    expect(stats.uniqueItems).toBe(2);
    expect(stats.mode).toBe('apple');
    expect(stats.count).toBe(3);
  });
});

describe('updateNumericalStats', () => {
  it('should update min, max, average, and count', () => {
    const stats: NumericalStats = { min: Infinity, max: -Infinity, average: 0, count: 0 };
    updateNumericalStats(stats, 10);
    updateNumericalStats(stats, 20);
    updateNumericalStats(stats, 5);

    expect(stats.min).toBe(5);
    expect(stats.max).toBe(20);
    expect(stats.average).toBeCloseTo(11.67, 2);
    expect(stats.count).toBe(3);
  });
});

describe('updateTemporalStats', () => {
  it('should update min, max, and range for temporal values', () => {
    const stats: TemporalStats = { min: Infinity, max: -Infinity, range: 0 };
    updateTemporalStats(stats, '2022-01-01');
    updateTemporalStats(stats, '2022-01-05');

    expect(stats.min).toBe(new Date('2022-01-01').getTime());
    expect(stats.max).toBe(new Date('2022-01-05').getTime());
    expect(stats.range).toBe(new Date('2022-01-05').getTime() - new Date('2022-01-01').getTime());
  });
});

describe('updateObjectStats', () => {
  it('should update the length of the object keys', () => {
    const stats: ObjectStats = { length: 0 };
    const value = { key1: 'value1', key2: 'value2' };
    updateObjectStats(stats, value);
    expect(stats.length).toBe(2);
  });
});

describe('initializeStatistics', () => {
  it('should initialize statistics for string type', () => {
    const stats = initializeStatistics('string');
    expect(stats).toEqual({ uniqueItems: 0, values: [], mode: '', count: 0 });
  });

  it('should initialize statistics for boolean type', () => {
    const stats = initializeStatistics('boolean');
    expect(stats).toEqual({ true: 0, false: 0 });
  });

  it('should initialize statistics for number type', () => {
    const stats = initializeStatistics('number');
    expect(stats).toEqual({ min: Infinity, max: -Infinity, average: 0, count: 0 });
  });

  it('should throw an error for an unknown type', () => {
    expect(() => initializeStatistics('unknown' as any)).toThrow('Unknown attribute type: unknown');
  });
});
